@@ -11,12 +11,12 @@ build_time_vars = {'ABIFLAGS': 'm',
  'AST_C_DIR': 'Python',
  'AST_H': 'Include/Python-ast.h',
  'AST_H_DIR': 'Include',
- 'BASECFLAGS': '-Wno-unused-result',
+ 'BASECFLAGS': '-Wno-unused-result -Wsign-compare -Wunreachable-code',
  'BASECPPFLAGS': '',
  'BASEMODLIBS': '',
- 'BINDIR': '/opt/lib/python-3.4.2/bin',
- 'BINLIBDEST': '/opt/lib/python-3.4.2/lib/python3.4',
- 'BLDLIBRARY': 'libpython3.4m.a',
+ 'BINDIR': '/opt/lib/python-3.5.1/bin',
+ 'BINLIBDEST': '/opt/lib/python-3.5.1/lib/python3.5',
+ 'BLDLIBRARY': 'libpython3.5m.a',
  'BLDSHARED': 'gcc -pthread -shared',
  'BUILDEXE': '',
  'BUILDPYTHON': 'python',
@@ -24,8 +24,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'BYTESTR_DEPS': '\\',
  'CC': 'gcc -pthread',
  'CCSHARED': '-fPIC',
- 'CFLAGS': '-Wno-unused-result -DNDEBUG -g -fwrapv -O3 -Wall '
-           '-Wstrict-prototypes',
+ 'CFLAGS': '-Wno-unused-result -Wsign-compare -Wunreachable-code -DNDEBUG -g '
+           '-fwrapv -O3 -Wall -Wstrict-prototypes',
  'CFLAGSFORSHARED': '',
  'CFLAGS_NODIST': '',
  'CONFIGFILES': 'configure configure.ac acconfig.h pyconfig.h.in '
@@ -34,25 +34,25 @@ build_time_vars = {'ABIFLAGS': 'm',
  'CONFIGURE_CFLAGS_NODIST': '-Werror=declaration-after-statement',
  'CONFIGURE_CPPFLAGS': '',
  'CONFIGURE_LDFLAGS': '',
- 'CONFIG_ARGS': "'--prefix=/opt/lib/python-3.4.2' '--enable-ipv6' "
+ 'CONFIG_ARGS': "'--prefix=/opt/lib/python-3.5.1' '--enable-ipv6' "
                 "'--enable-loadable-sqlite-extensions' "
                 "'--with-dbmliborder=bdb' '--with-wide-unicode' "
                 "'--with-computed-gotos' '--with-pymalloc'",
- 'CONFINCLUDEDIR': '/opt/lib/python-3.4.2/include',
- 'CONFINCLUDEPY': '/opt/lib/python-3.4.2/include/python3.4m',
+ 'CONFINCLUDEDIR': '/opt/lib/python-3.5.1/include',
+ 'CONFINCLUDEPY': '/opt/lib/python-3.5.1/include/python3.5m',
  'COREPYTHONPATH': ':plat-linux',
- 'COVERAGE_INFO': '/home/sources/packages/Python-3.4.2/coverage.info',
- 'COVERAGE_REPORT': '/home/sources/packages/Python-3.4.2/lcov-report',
+ 'COVERAGE_INFO': '/home/sources/packages/Python-3.5.1/coverage.info',
+ 'COVERAGE_REPORT': '/home/sources/packages/Python-3.5.1/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
  'CPPFLAGS': '-I. -IInclude -I./Include',
  'CXX': 'g++ -pthread',
- 'DESTDIRS': '/opt/lib/python-3.4.2 /opt/lib/python-3.4.2/lib '
-             '/opt/lib/python-3.4.2/lib/python3.4 '
-             '/opt/lib/python-3.4.2/lib/python3.4/lib-dynload',
- 'DESTLIB': '/opt/lib/python-3.4.2/lib/python3.4',
+ 'DESTDIRS': '/opt/lib/python-3.5.1 /opt/lib/python-3.5.1/lib '
+             '/opt/lib/python-3.5.1/lib/python3.5 '
+             '/opt/lib/python-3.5.1/lib/python3.5/lib-dynload',
+ 'DESTLIB': '/opt/lib/python-3.5.1/lib/python3.5',
  'DESTPATH': '',
- 'DESTSHARED': '/opt/lib/python-3.4.2/lib/python3.4/lib-dynload',
+ 'DESTSHARED': '/opt/lib/python-3.5.1/lib/python3.5/lib-dynload',
  'DIRMODE': 755,
  'DIST': 'README ChangeLog configure configure.ac acconfig.h pyconfig.h.in '
          'Makefile.pre.in Include Lib Misc Ext-dummy',
@@ -70,10 +70,9 @@ build_time_vars = {'ABIFLAGS': 'm',
  'EXE': '',
  'EXEMODE': 755,
  'EXTRAMACHDEPPATH': '',
- 'EXTRAPLATDIR': '@EXTRAPLATDIR@',
  'EXTRATESTOPTS': '',
  'EXTRA_CFLAGS': '',
- 'EXT_SUFFIX': '.cpython-34m.so',
+ 'EXT_SUFFIX': '.cpython-35m-i386-linux-gnu.so',
  'FILEMODE': 644,
  'FLOCK_NEEDS_LIBBSD': 0,
  'GETPGRP_HAVE_ARG': 0,
@@ -103,6 +102,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_BROKEN_PTHREAD_SIGMASK': 0,
  'HAVE_BROKEN_SEM_GETVALUE': 0,
  'HAVE_BROKEN_UNSETENV': 0,
+ 'HAVE_BUILTIN_ATOMIC': 1,
  'HAVE_C99_BOOL': 1,
  'HAVE_CHFLAGS': 0,
  'HAVE_CHOWN': 1,
@@ -128,6 +128,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_DEV_PTC': 0,
  'HAVE_DEV_PTMX': 1,
  'HAVE_DIRECT_H': 0,
+ 'HAVE_DIRENT_D_TYPE': 1,
  'HAVE_DIRENT_H': 1,
  'HAVE_DIRFD': 1,
  'HAVE_DLFCN_H': 1,
@@ -172,11 +173,13 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_FUTIMESAT': 1,
  'HAVE_GAI_STRERROR': 1,
  'HAVE_GAMMA': 1,
+ 'HAVE_GCC_ASM_FOR_MC68881': 0,
  'HAVE_GCC_ASM_FOR_X64': 0,
  'HAVE_GCC_ASM_FOR_X87': 1,
  'HAVE_GCC_UINT128_T': 0,
  'HAVE_GETADDRINFO': 1,
  'HAVE_GETC_UNLOCKED': 1,
+ 'HAVE_GETENTROPY': 0,
  'HAVE_GETGROUPLIST': 1,
  'HAVE_GETGROUPS': 1,
  'HAVE_GETHOSTBYNAME': 0,
@@ -195,6 +198,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_GETPID': 1,
  'HAVE_GETPRIORITY': 1,
  'HAVE_GETPWENT': 1,
+ 'HAVE_GETRANDOM': 0,
+ 'HAVE_GETRANDOM_SYSCALL': 0,
  'HAVE_GETRESGID': 1,
  'HAVE_GETRESUID': 1,
  'HAVE_GETSID': 1,
@@ -238,6 +243,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_LINKAT': 1,
  'HAVE_LINUX_CAN_BCM_H': 0,
  'HAVE_LINUX_CAN_H': 1,
+ 'HAVE_LINUX_CAN_RAW_FD_FRAMES': 0,
  'HAVE_LINUX_CAN_RAW_H': 1,
  'HAVE_LINUX_NETLINK_H': 1,
  'HAVE_LINUX_TIPC_H': 1,
@@ -290,11 +296,13 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_PTY_H': 1,
  'HAVE_PUTENV': 1,
  'HAVE_PWRITE': 1,
+ 'HAVE_RAND_EGD': 1,
  'HAVE_READLINK': 1,
  'HAVE_READLINKAT': 1,
  'HAVE_READV': 1,
  'HAVE_REALPATH': 1,
  'HAVE_RENAMEAT': 1,
+ 'HAVE_RL_APPEND_HISTORY': 1,
  'HAVE_RL_CALLBACK': 1,
  'HAVE_RL_CATCH_SIGNAL': 1,
  'HAVE_RL_COMPLETION_APPEND_CHARACTER': 1,
@@ -354,6 +362,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_STDARG_PROTOTYPES': 1,
  'HAVE_STDINT_H': 1,
  'HAVE_STDLIB_H': 1,
+ 'HAVE_STD_ATOMIC': 0,
  'HAVE_STRDUP': 1,
  'HAVE_STRFTIME': 1,
  'HAVE_STRINGS_H': 1,
@@ -451,58 +460,61 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HGTAG': '',
  'HGVERSION': '',
  'HOST_GNU_TYPE': 'i686-pc-linux-gnu',
- 'INCLDIRSTOMAKE': '/opt/lib/python-3.4.2/include '
-                   '/opt/lib/python-3.4.2/include '
-                   '/opt/lib/python-3.4.2/include/python3.4m '
-                   '/opt/lib/python-3.4.2/include/python3.4m',
- 'INCLUDEDIR': '/opt/lib/python-3.4.2/include',
- 'INCLUDEPY': '/opt/lib/python-3.4.2/include/python3.4m',
+ 'INCLDIRSTOMAKE': '/opt/lib/python-3.5.1/include '
+                   '/opt/lib/python-3.5.1/include '
+                   '/opt/lib/python-3.5.1/include/python3.5m '
+                   '/opt/lib/python-3.5.1/include/python3.5m',
+ 'INCLUDEDIR': '/opt/lib/python-3.5.1/include',
+ 'INCLUDEPY': '/opt/lib/python-3.5.1/include/python3.5m',
  'INSTALL': '/usr/bin/install -c',
  'INSTALL_DATA': '/usr/bin/install -c -m 644',
  'INSTALL_PROGRAM': '/usr/bin/install -c',
  'INSTALL_SCRIPT': '/usr/bin/install -c',
  'INSTALL_SHARED': '/usr/bin/install -c -m 555',
- 'INSTSONAME': 'libpython3.4m.a',
+ 'INSTSONAME': 'libpython3.5m.a',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
  'LDCXXSHARED': 'g++ -pthread -shared',
  'LDFLAGS': '',
  'LDLAST': '',
- 'LDLIBRARY': 'libpython3.4m.a',
+ 'LDLIBRARY': 'libpython3.5m.a',
  'LDLIBRARYDIR': '',
  'LDSHARED': 'gcc -pthread -shared',
- 'LDVERSION': '3.4m',
+ 'LDVERSION': '3.5m',
  'LIBC': '',
- 'LIBDEST': '/opt/lib/python-3.4.2/lib/python3.4',
- 'LIBDIR': '/opt/lib/python-3.4.2/lib',
+ 'LIBDEST': '/opt/lib/python-3.5.1/lib/python3.5',
+ 'LIBDIR': '/opt/lib/python-3.5.1/lib',
  'LIBFFI_INCLUDEDIR': '',
  'LIBM': '-lm',
  'LIBOBJDIR': 'Python/',
  'LIBOBJS': '',
- 'LIBPC': '/opt/lib/python-3.4.2/lib/pkgconfig',
- 'LIBPL': '/opt/lib/python-3.4.2/lib/python3.4/config-3.4m',
- 'LIBRARY': 'libpython3.4m.a',
+ 'LIBPC': '/opt/lib/python-3.5.1/lib/pkgconfig',
+ 'LIBPL': '/opt/lib/python-3.5.1/lib/python3.5/config-3.5m',
+ 'LIBRARY': 'libpython3.5m.a',
  'LIBRARY_OBJS': '\\',
  'LIBRARY_OBJS_OMIT_FROZEN': '\\',
- 'LIBS': '-lpthread -ldl  -lutil',
+ 'LIBS': '-lpthread -ldl  -lutil -lrt',
  'LIBSUBDIRS': 'tkinter tkinter/test tkinter/test/test_tkinter \\',
  'LINKCC': 'gcc -pthread',
  'LINKFORSHARED': '-Xlinker -export-dynamic',
  'LIPO_32BIT_FLAGS': '',
+ 'LLVM_PROF_ERR': 'no',
+ 'LLVM_PROF_FILE': '',
+ 'LLVM_PROF_MERGER': 'true',
  'LN': 'ln',
  'LOCALMODLIBS': '',
  'LOG1P_DROPS_ZERO_SIGN': 0,
  'MACHDEP': 'linux',
  'MACHDEPPATH': ':plat-linux',
- 'MACHDEPS': 'plat-linux @EXTRAPLATDIR@',
+ 'MACHDEPS': 'plat-linux',
  'MACHDEP_OBJS': '',
- 'MACHDESTLIB': '/opt/lib/python-3.4.2/lib/python3.4',
+ 'MACHDESTLIB': '/opt/lib/python-3.5.1/lib/python3.5',
  'MACOSX_DEPLOYMENT_TARGET': '',
  'MAINCC': 'gcc -pthread',
  'MAJOR_IN_MKDEV': 0,
  'MAJOR_IN_SYSMACROS': 0,
  'MAKESETUP': './Modules/makesetup',
- 'MANDIR': '/opt/lib/python-3.4.2/share/man',
+ 'MANDIR': '/opt/lib/python-3.5.1/share/man',
  'MKDIR_P': '/bin/mkdir -p',
  'MODLIBS': '',
  'MODOBJS': 'Modules/_threadmodule.o  Modules/signalmodule.o  '
@@ -511,10 +523,10 @@ build_time_vars = {'ABIFLAGS': 'm',
             'Modules/_weakref.o  Modules/_functoolsmodule.o  '
             'Modules/_operator.o  Modules/_collectionsmodule.o  '
             'Modules/itertoolsmodule.o  Modules/atexitmodule.o  '
-            'Modules/_stat.o  Modules/_localemodule.o  Modules/_iomodule.o '
-            'Modules/iobase.o Modules/fileio.o Modules/bytesio.o '
-            'Modules/bufferedio.o Modules/textio.o Modules/stringio.o  '
-            'Modules/zipimport.o  Modules/faulthandler.o  '
+            'Modules/_stat.o  Modules/timemodule.o  Modules/_localemodule.o  '
+            'Modules/_iomodule.o Modules/iobase.o Modules/fileio.o '
+            'Modules/bytesio.o Modules/bufferedio.o Modules/textio.o '
+            'Modules/stringio.o  Modules/zipimport.o  Modules/faulthandler.o  '
             'Modules/_tracemalloc.o Modules/hashtable.o  '
             'Modules/symtablemodule.o  Modules/xxsubtype.o',
  'MODULE_OBJS': '\\',
@@ -525,6 +537,11 @@ build_time_vars = {'ABIFLAGS': 'm',
  'OPCODETARGETGEN': '\\',
  'OPCODETARGETGEN_FILES': '\\',
  'OPCODETARGETS_H': '\\',
+ 'OPCODE_H': './Include/opcode.h',
+ 'OPCODE_H_DIR': './Include',
+ 'OPCODE_H_GEN': 'python  ./Tools/scripts/generate_opcode_h.py ./Lib/opcode.py '
+                 './Include/opcode.h',
+ 'OPCODE_H_SCRIPT': './Tools/scripts/generate_opcode_h.py',
  'OPT': '-DNDEBUG -g -fwrapv -O3 -Wall -Wstrict-prototypes',
  'OTHER_LIBTOOL_OPT': '',
  'PACKAGE_BUGREPORT': 0,
@@ -539,11 +556,13 @@ build_time_vars = {'ABIFLAGS': 'm',
  'PGENOBJS': '\\ \\',
  'PGENSRCS': '\\ \\',
  'PGOBJS': '\\',
+ 'PGO_PROF_GEN_FLAG': '-fprofile-generate',
+ 'PGO_PROF_USE_FLAG': '-fprofile-use -fprofile-correction',
  'PGSRCS': '\\',
  'PLATDIR': 'plat-linux',
  'POBJS': '\\',
  'POSIX_SEMAPHORES_NOT_ENABLED': 0,
- 'PROFILE_TASK': './Tools/pybench/pybench.py -n 2 --with-gc --with-syscheck',
+ 'PROFILE_TASK': '-m test.regrtest --pgo',
  'PSRCS': '\\',
  'PTHREAD_SYSTEM_SCHED_SUPPORTED': 1,
  'PURIFY': '',
@@ -558,12 +577,13 @@ build_time_vars = {'ABIFLAGS': 'm',
  'PYTHON_FOR_BUILD': './python -E',
  'PYTHON_HEADERS': '\\',
  'PYTHON_OBJS': '\\',
- 'PY_CFLAGS': '-Wno-unused-result -DNDEBUG -g -fwrapv -O3 -Wall '
-              '-Wstrict-prototypes',
+ 'PY_CFLAGS': '-Wno-unused-result -Wsign-compare -Wunreachable-code -DNDEBUG '
+              '-g -fwrapv -O3 -Wall -Wstrict-prototypes',
  'PY_CFLAGS_NODIST': '-Werror=declaration-after-statement',
- 'PY_CORE_CFLAGS': '-Wno-unused-result -DNDEBUG -g -fwrapv -O3 -Wall '
-                   '-Wstrict-prototypes -Werror=declaration-after-statement '
-                   '-I. -IInclude -I./Include  -DPy_BUILD_CORE',
+ 'PY_CORE_CFLAGS': '-Wno-unused-result -Wsign-compare -Wunreachable-code '
+                   '-DNDEBUG -g -fwrapv -O3 -Wall -Wstrict-prototypes '
+                   '-Werror=declaration-after-statement -I. -IInclude '
+                   '-I./Include  -DPy_BUILD_CORE',
  'PY_CPPFLAGS': '-I. -IInclude -I./Include',
  'PY_FORMAT_LONG_LONG': '"ll"',
  'PY_FORMAT_SIZE_T': '"z"',
@@ -577,11 +597,11 @@ build_time_vars = {'ABIFLAGS': 'm',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
  'RUNSHARED': '',
- 'SCRIPTDIR': '/opt/lib/python-3.4.2/lib',
+ 'SCRIPTDIR': '/opt/lib/python-3.5.1/lib',
  'SETPGRP_HAVE_ARG': 0,
  'SGI_ABI': '',
  'SHELL': '/bin/sh',
- 'SHLIBS': '-lpthread -ldl  -lutil',
+ 'SHLIBS': '-lpthread -ldl  -lutil -lrt',
  'SHLIB_SUFFIX': '.so',
  'SIGNAL_OBJS': '',
  'SIGNED_RIGHT_SHIFT_ZERO_FILLS': 0,
@@ -603,8 +623,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'SIZEOF_VOID_P': 4,
  'SIZEOF_WCHAR_T': 4,
  'SIZEOF__BOOL': 1,
- 'SOABI': 'cpython-34m',
- 'SRCDIRS': 'Parser Grammar Objects Python Modules Mac',
+ 'SOABI': 'cpython-35m-i386-linux-gnu',
+ 'SRCDIRS': 'Parser Grammar Objects Python Modules Mac Programs',
  'SRC_GDB_HOOKS': './Tools/gdb/libpython.py',
  'STDC_HEADERS': 1,
  'STRICT_SYSV_CURSES': "/* Don't use ncurses extensions */",
@@ -631,7 +651,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'USE_COMPUTED_GOTOS': 1,
  'USE_INLINE': 1,
  'VA_LIST_IS_ARRAY': 0,
- 'VERSION': '3.4',
+ 'VERSION': '3.5',
  'WANT_SIGFPE_HANDLER': 0,
  'WINDOW_HAS_FLAGS': 1,
  'WITH_DOC_STRINGS': 1,
@@ -644,9 +664,9 @@ build_time_vars = {'ABIFLAGS': 'm',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 1,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/home/sources/packages/Python-3.4.2',
- 'abs_srcdir': '/home/sources/packages/Python-3.4.2',
- 'datarootdir': '/opt/lib/python-3.4.2/share',
- 'exec_prefix': '/opt/lib/python-3.4.2',
- 'prefix': '/opt/lib/python-3.4.2',
+ 'abs_builddir': '/home/sources/packages/Python-3.5.1',
+ 'abs_srcdir': '/home/sources/packages/Python-3.5.1',
+ 'datarootdir': '/opt/lib/python-3.5.1/share',
+ 'exec_prefix': '/opt/lib/python-3.5.1',
+ 'prefix': '/opt/lib/python-3.5.1',
  'srcdir': '.'}
